package com.hp.user.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import com.hp.user.client.entity.*;
import com.hp.user.client.service.CustomerCompanyService;
import com.hp.user.service.dao.CustomerCompanyMapper;
import com.hp.user.service.dao.CustomerServiceProviderEnterpriseMapper;
import com.hp.user.service.entity.CompanyAudit;
import com.hp.user.service.entity.CustomerCompany;
import com.hp.user.service.entity.CustomerServiceProviderEnterprise;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 企业/公司 服务实现类
 * </p>
 *
 * @author 王怡静
 * @since 2020-11-20
 */
@Service
public class CustomerCompanyServiceImpl implements CustomerCompanyService {
    @Autowired
    private CustomerCompanyMapper customerCompanyMapper;
    @Autowired
    private CustomerServiceProviderEnterpriseMapper customerServiceProviderEnterpriseMapper;

    @Override
    public boolean insert(CustomerCompanyVO company) {
        CustomerCompany temp = new CustomerCompany();
        BeanUtils.copyProperties(company, temp);
        // 判断数据库操作是否成功
        return SqlHelper.retBool(customerCompanyMapper.insert(temp));
    }

    @Override
    public boolean delete(String companyId) {
        CustomerCompany temp = new CustomerCompany();
        try {
            // 将deleted字段设置为1
            temp = customerCompanyMapper.selectById(companyId);
            temp.setSysDeleted((short) 1);
        } catch (Exception e) {

        }
        // 判断数据库操作是否成功
        return SqlHelper.retBool(customerCompanyMapper.updateById(temp));
    }

    @Override
    public boolean update(CustomerCompanyVO company) {
        CustomerCompany temp = new CustomerCompany();
        BeanUtils.copyProperties(company, temp);
        // 判断数据库操作是否成功
        return SqlHelper.retBool(customerCompanyMapper.updateById(temp));
    }

    @Override
    public CustomerCompanyVO queryCompanyByCompanyId(String companyId) {
        CustomerCompanyVO customerCompanyVO = new CustomerCompanyVO();
        CustomerCompany temp = customerCompanyMapper.selectById(companyId);
        // 这个已经被删除
        if (temp == null || temp.getSysDeleted() == 1) return null;
        else {
            BeanUtils.copyProperties(temp, customerCompanyVO);
            return customerCompanyVO;
        }
    }

    @Override
    public CompanyPage getCompanysPage(Long current, Integer size, Integer type) {
        CompanyPage companyPage = new CompanyPage();
        List<CustomerCompanyVO> companyVOs = new ArrayList<>();
        IPage<CustomerCompany> page = new Page<>(current, size);
        QueryWrapper<CustomerCompany> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("sys_create_time");
        queryWrapper.eq("sys_deleted", 0);
        queryWrapper.eq("type", type);
        customerCompanyMapper.selectPage(page, queryWrapper);
        List<CustomerCompany> companys = page.getRecords();
        CustomerCompanyVO temp;
        for (CustomerCompany company : companys) {
            temp = new CustomerCompanyVO();
            BeanUtils.copyProperties(company, temp);
            companyVOs.add(temp);
        }
        companyPage.setCompanyList(companyVOs);
        companyPage.setTotal(page.getTotal());
        companyPage.setCurrent(current);
        companyPage.setSize(size);
        return companyPage;
    }

    @Override
    public CompanyPage queryServiceProvidersByName(String name, Long current, Integer size) {
        CompanyPage companyPage = new CompanyPage();
        List<CustomerCompanyVO> companyVOs = new ArrayList<>();
        IPage<CustomerCompany> page = new Page<>(current, size);
        QueryWrapper<CustomerCompany> wrapper = new QueryWrapper<>();
        wrapper.orderByDesc("sys_create_time");
        wrapper.eq("sys_deleted", 0);
        wrapper.eq("type", 0);
        if (name != null) {
            wrapper.like("name", name);
        }
        customerCompanyMapper.selectPage(page, wrapper);
        List<CustomerCompany> companys = page.getRecords();
        CustomerCompanyVO temp;
        for (CustomerCompany company : companys) {
            temp = new CustomerCompanyVO();
            BeanUtils.copyProperties(company, temp);
            companyVOs.add(temp);
        }
        companyPage.setCompanyList(companyVOs);
        companyPage.setTotal(page.getTotal());
        companyPage.setCurrent(current);
        companyPage.setSize(size);
        return companyPage;
    }

    @Override
    public CompanyPage queryCompanysPageByNameAndServiceAndStatus(String name, String serviceProviderName, Integer status, Long current, Integer size) {
        CompanyPage companyPage = new CompanyPage();
        List<CustomerCompanyVO> companyVOs = new ArrayList<>();
        IPage<CustomerCompany> page = new Page<>(current, size);
        QueryWrapper<CustomerCompany> wrapper = new QueryWrapper<>();
        // 关联服务商
        if (!StringUtils.isEmpty(serviceProviderName)) {
            QueryWrapper<CustomerCompany> queryWrapper = new QueryWrapper<>();
            // 先查出服务商信息，再根据服务商Id与企业Id的对应情况查企业
            queryWrapper.like("name", serviceProviderName);
            String serviceProviderId = customerCompanyMapper.selectOne(queryWrapper).getCustomerId();
            //根据服务商Id与企业Id的对应情况查企业Id
            QueryWrapper<CustomerServiceProviderEnterprise> tempWrapper = new QueryWrapper<>();
            tempWrapper.eq("service_provider_id", serviceProviderId);
            List<CustomerServiceProviderEnterprise> temp = customerServiceProviderEnterpriseMapper.selectList(tempWrapper);
            List<String> companyIds = new ArrayList<>();
            for (CustomerServiceProviderEnterprise tt : temp) {
                companyIds.add(tt.getEnterpriseId());
            }
            wrapper.in("customer_id", companyIds);
        }
        if (status != 0) {
            wrapper.eq("status", status);
        }
        if (!StringUtils.isEmpty(name)) {
            wrapper.like("name", name);
        }
        wrapper.orderByDesc("sys_create_time");
        wrapper.eq("sys_deleted", 0);
        wrapper.eq("type", 1);
        customerCompanyMapper.selectPage(page, wrapper);
        List<CustomerCompany> companys = page.getRecords();
        CustomerCompanyVO t;
        for (CustomerCompany company : companys) {
            t = new CustomerCompanyVO();
            BeanUtils.copyProperties(company, t);
            companyVOs.add(t);
        }
        companyPage.setCompanyList(companyVOs);
        companyPage.setTotal(page.getTotal());
        companyPage.setCurrent(current);
        companyPage.setSize(size);
        return companyPage;
    }

    @Override
    public CompanyAuditPage getCompanyAuditsPage(Long current, Integer size) {
        CompanyAuditPage companyAuditPage = new CompanyAuditPage();
        Page<CompanyAudit> page = new Page<>(current, size);
        List<CompanyAudit> companyAudits = customerCompanyMapper.selectCompanyAuditPage(page);

        List<CompanyAuditVO> companyAuditVOList = new ArrayList<>();
        CompanyAuditVO temp;
        for (CompanyAudit companyAudit : companyAudits) {
            temp = new CompanyAuditVO();
            BeanUtils.copyProperties(companyAudit, temp);
            companyAuditVOList.add(temp);
        }
        companyAuditPage.setCompanyAuditList(companyAuditVOList);
        companyAuditPage.setTotal(page.getTotal());
        companyAuditPage.setCurrent(current);
        companyAuditPage.setSize(size);
        return companyAuditPage;
    }

    @Override
    public CompanyAuditPage queryCompanysPageByNameAndStatusAndCreateTime(String name, Integer status, Date startTime, Date endTime, Long current, Integer size) {
        CompanyAuditPage companyAuditPage = new CompanyAuditPage();
        Page<CompanyAudit> page = new Page<>(current, size);
        List<CompanyAudit> companyAudits = customerCompanyMapper.queryCompanyAuditPageByNameAndStatusAndCreateTime(page, name, status, startTime, endTime);

        List<CompanyAuditVO> companyAuditVOList = new ArrayList<>();
        CompanyAuditVO temp;
        for (CompanyAudit companyAudit : companyAudits) {
            temp = new CompanyAuditVO();
            BeanUtils.copyProperties(companyAudit, temp);
            companyAuditVOList.add(temp);
        }
        companyAuditPage.setCompanyAuditList(companyAuditVOList);
        companyAuditPage.setTotal(page.getTotal());
        companyAuditPage.setCurrent(current);
        companyAuditPage.setSize(size);
        return companyAuditPage;
    }
}
